﻿// .Net Follower http://dotnetfollower.com

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace Upgrade2007BdcModelTo2010
{
    public partial class Form1 : Form
    {
        private bool upgradeStarted = false;

        public Form1()
        {
            InitializeComponent();
            sourceListBox_SelectedIndexChanged(null, null);
            UpdateLabels();
        }

        private void addBtn_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
                foreach (string filePath in openFileDialog1.FileNames)
                {
                    FileListItem fileListItem = new FileListItem() { FilePath = filePath };
                    if (!sourceListBox.Items.Contains(fileListItem))
                        sourceListBox.Items.Add(fileListItem);

                    UpdateLabels();
                }
        }

        private void RemBtn_Click(object sender, EventArgs e)
        {
            if (sourceListBox.SelectedIndices.Count > 0)
            {
                sourceListBox.Items.RemoveRange(sourceListBox.SelectedItems);
                UpdateLabels();
            }
        }

        private void sourceListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!upgradeStarted)
                remBtn.Enabled = sourceListBox.SelectedIndices.Count > 0;
        }

        private void upgradeBtn_Click(object sender, EventArgs e)
        {
            UpdateProgressBar(0);
            resultListBox.Items.Clear();
            UpdateLabels();
            EnableButtons(false);
            upgradeStarted = true;

            ThreadPool.QueueUserWorkItem(delegate
                {
                    List<string> sourceFilePathList = sourceListBox.Items.ToList().ConvertAll(obj => ((FileListItem) obj).FilePath);
                    Upgrade(sourceFilePathList.ToArray(), "c:\\output");
                });
        }        

        private void Upgrade(string[] sourcefilePaths, string outputDir)
        {
            for (int i = 0; i < sourcefilePaths.Length; i ++ )                
            {
                Exception exception  = null;
                string    resultFile = null;

                try
                {
                    string modelText = File.ReadAllText(sourcefilePaths[i]);

                    Sp07BdcModelToSp10Converter converter = new Sp07BdcModelToSp10Converter();
                    string res = converter.Convert(modelText);

                    Directory.CreateDirectory(outputDir);
                    resultFile = outputDir + @"\" + Path.GetFileName(sourcefilePaths[i]);
                    File.WriteAllText(resultFile, res);
                }
                catch (Exception ex)
                {
                    exception = ex;
                }

                UpdateProgressBar((i + 1) * 100 / sourcefilePaths.Length);
                AddResultListItem(exception != null ? new ErrorListItem(exception, sourcefilePaths[i]) : new FileListItem() { FilePath = resultFile });
            }

            UpdateProgressBar(100);
            EnableButtons(true);
            upgradeStarted = false;
        }

        private void UpdateProgressBar(int currentValue)
        {
            // if it's UI thread, simply update ProgressBar as usual. 
            // Otherwise, make the current method be called in UI thread.
            if (progressBar1.InvokeRequired)
                progressBar1.BeginInvoke(new Action(() => UpdateProgressBar(currentValue)));
            else
                progressBar1.Value = currentValue;
        }

        private void AddResultListItem(FileListItem item)
        {
            if (resultListBox.InvokeRequired)
                resultListBox.BeginInvoke(new Action(() => AddResultListItem(item)));
            else
            {
                resultListBox.Items.Add(item);
                UpdateLabels();
            }
        }

        private void listBox_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            ListBox listBox = (ListBox)sender;
            int clickedIndex = listBox.IndexFromPoint(e.X, e.Y);
            if(clickedIndex >= 0 && clickedIndex < listBox.Items.Count)
            {
                ErrorListItem errorListItem = listBox.Items[clickedIndex] as ErrorListItem;
                if (errorListItem != null)
                {
                    OverviewForm overviewForm = new OverviewForm();

                    overviewForm.webBrowser1.Visible      = false;
                    overviewForm.textBox1.Visible         = true;                    
                    overviewForm.textBox1.Text            = errorListItem.GetDetails();
                    overviewForm.textBox1.SelectionLength = 0;
                    overviewForm.textBox1.SelectionStart  = 0;                    
                    overviewForm.Text                     = "Error on file upgrading - " + errorListItem;

                    overviewForm.Show();
                }
                else
                {
                    FileListItem fileListItem = listBox.Items[clickedIndex] as FileListItem;
                    if (fileListItem != null)
                    {
                        OverviewForm overviewForm = new OverviewForm();

                        overviewForm.webBrowser1.Visible = true;
                        overviewForm.textBox1.Visible    = false;
                        overviewForm.webBrowser1.Url     = new Uri(fileListItem.FilePath);
                        overviewForm.Text                = fileListItem.FilePath;

                        overviewForm.Show();
                    }
                }
            }
        }

        private void resultListBox_DrawItem(object sender, DrawItemEventArgs e)
        {
            object listItem = ((ListBox)sender).Items[e.Index];            
            Brush  brush    = listItem is ErrorListItem ? Brushes.Red : new SolidBrush(e.ForeColor);
            Font font       = listItem is ErrorListItem ? new Font(e.Font, FontStyle.Bold) : e.Font;
            e.DrawBackground();
            e.Graphics.DrawString(listItem.ToString(), font, brush, e.Bounds, StringFormat.GenericDefault);
        }
        
        private void listBox_MouseMove(object sender, MouseEventArgs e)
        {
            ListBox lb = (ListBox)sender;
            int index = lb.IndexFromPoint(e.Location);

            if (index >=0 && index < lb.Items.Count)
            {
                FileListItem listItem = (FileListItem)lb.Items[index];

                string toolTipString = listItem.FilePath;
                if (listItem is ErrorListItem)
                    toolTipString = listItem.ToString();

                // check if tooltip text coincides with the current one,
                // if so, do nothing
                if (toolTip1.GetToolTip(lb) != toolTipString)
                    toolTip1.SetToolTip(lb, toolTipString);
            }
            else
                toolTip1.Hide(lb);
        }        

        private void UpdateLabels()
        {
            sourceAmntLabel.Text      = "Files: " + sourceListBox.Items.Count;
            int errorsAmount          = resultListBox.Items.ToList().Count(obj => obj is ErrorListItem);
            upgradedSuccessLabel.Text = "Successfully upgraded: " + (resultListBox.Items.Count - errorsAmount);
            errorAmntLabel.Text       = "Errors: " + errorsAmount; ;
        }

        private void EnableButtons(bool enable)
        {
            if (addBtn.InvokeRequired)
                addBtn.BeginInvoke(new Action(() => EnableButtons(enable)));
            else
            {
                addBtn.Enabled     = enable;
                upgradeBtn.Enabled = enable;

                if (enable)
                    remBtn.Enabled = sourceListBox.SelectedIndices.Count > 0 ? true : false;
                else
                    remBtn.Enabled = false;
            }
        }
    }
}
