﻿// .Net Follower http://dotnetfollower.com

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Upgrade2007BdcModelTo2010
{
    public class FileListItem
    {
        public string FilePath { get; set; }

        public override string ToString()
        {
            // return file name without directory path
            return Path.GetFileName(FilePath);
        }

        public override bool Equals(object obj)
        {
            FileListItem fileListItem = obj as FileListItem;

            if (fileListItem == null)
                return false;

            if (object.ReferenceEquals(this, fileListItem))
                return true;

            return string.Equals(this.FilePath, fileListItem.FilePath, StringComparison.OrdinalIgnoreCase);            
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

    public class ErrorListItem : FileListItem
    {
        protected string _message = null;

        public string Message
        {
            get { return (string.IsNullOrEmpty(_message) && Exception != null) ? Exception.Message : string.Empty; }
            set { _message = value; }
        }
        public Exception Exception { get; set; }

        public ErrorListItem(Exception ex, string filePath)
        {
            Exception = ex;
            FilePath  = filePath;
        }
        public ErrorListItem(string message, string filePath)
        {
            _message = message;
            FilePath = filePath;
        }

        public override string ToString()
        {
            return string.Format("{0} ({1})", Path.GetFileName(FilePath), Message);
        }

        public string GetDetails()
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine(Message).AppendLine();
            if (Exception != null)
            {
                if(Exception.InnerException != null)
                    sb.AppendLine(string.Format("[Inner exception: {0}]", Exception.InnerException.Message)).AppendLine();
                sb.AppendLine(string.Format("[Stack trace: {0}]", Exception.StackTrace));
            }

            return sb.ToString();
        }
    }
}
